"""This is the main module of the package using agent tool calling"""

import torch
from gnais.agent.agent import *

QUERY = os.getenv("QUERY")
if QUERY is None:
    raise ValueError("QUERY must be specified for program to run")
SEED = os.getenv("SEED")
if SEED is None:
    raise ValueError("SEED must be specified for reproducibility")
MODEL_NAME = os.getenv("MODEL_NAME")
if MODEL_NAME is None:
    raise ValueError("MODEL_NAME must be specified - either proprietary or local")
MODEL_TYPE = os.getenv("MODEL_TYPE")
if MODEL_TYPE is None:
    raise ValueError("MODEL_TYPE must be specified")


torch.manual_seed(SEED)

if torch.cuda.is_available():
    torch.cuda.manual_seed_all(SEED)


if int(MODEL_TYPE) == 0:
    llm = dspy.LM(
        model=f"openai/{MODEL_NAME}",
        api_base="http://localhost:7501/v1",
        api_key="local",
        model_type="chat",
        max_tokens=10_000,
        n_ctx=10_000,
        seed=2_025,
        temperature=0,
        verbose=False,
    )
elif int(MODEL_TYPE) == 1:
    API_KEY = os.getenv("API_KEY")
    if API_KEY is None:
        raise ValueError("Valid API_KEY must be specified to use the proprietary model")
    llm = dspy.LM(
        MODEL_NAME,
        api_key=API_KEY,
        max_tokens=10_000,
        temperature=0,
        verbose=False,
    )
else:
    raise ValueError("MODEL_TYPE must be 0 or 1")


dspy.configure(lm=llm, adapter=dspy.JSONAdapter())


def search(query: str):
    system_prompt = """
            You excel at addressing search query using the context you have. You do not make mistakes.
            Extract answers to the query from the context and provide links associated with each RDF entity.
            All links pointing to specific traits should be translated to CD links using the trait id (numeric code) and the dataset name specifically.
            Original trait link: https://rdf.genenetwork.org/v1/id/trait_BXDPublish_16339
            Trait id: 16339
            Dataset name: BXDPublish
            New trait link: https://cd.genenetwork.org/show_trait?trait_id=16339&dataset=BXDPublish
            \n
            """
    final_query = system_prompt + query
    search = AISearch()
    output = search(query=final_query).get("solution")
    return output.model_dump_json(indent=4)



__all__ = (
    "Config",
)
import os
from dotenv import load_dotenv

load_dotenv()


class Config:
    CORPUS_PATH = os.environ.get("CORPUS_PATH")
    if CORPUS_PATH is None:
        raise RuntimeError("CORPUS_PATH is not set")

    PCORPUS_PATH = os.environ.get("PCORPUS_PATH")
    if PCORPUS_PATH is None:
        raise RuntimeError("PCORPUS_PATH is not set")

    DB_PATH = os.environ.get("DB_PATH")
    if DB_PATH is None:
        raise RuntimeError("DB_PATH is not set")

    SEED = os.environ.get("SEED")
    if SEED is None:
        raise RuntimeError("SEED is not set")

    MODEL_NAME = os.environ.get("MODEL_NAME")
    if MODEL_NAME is None:
        raise RuntimeError("MODEL_NAME is not set")

    MODEL_TYPE = int(os.environ.get("MODEL_TYPE"))
    if MODEL_TYPE is None:
        raise RuntimeError("MODEL_TYPE is not set")
    if MODEL_TYPE not in (0, 1,):
        raise ValueError("MODEL_TYPE must be 0 or 1")

    API_KEY = os.environ.get("API_KEY")
    if MODEL_TYPE and API_KEY is None:
        raise RuntimeError("API_KEY is not set")



"""Agent with sparql tool calling for AI search in GeneNetwork"""

import os
from typing import Any

import dspy
from gnais.rag.config import ListInformation
from SPARQLWrapper import JSON, SPARQLWrapper

TTL_PATH = os.getenv("TTL_PATH")
if TTL_PATH is None:
    raise FileNotFoundError(
        "TTL_PATH must be specified to extract RDF schema and build queries"
    )


def extract_schema(ttl_path: str) -> tuple[list, list]:
    ttl_files = [
        os.path.join(ttl_path, ttl) for ttl in os.listdir(ttl_path) if "ttl" in ttl
    ]
    prefixes = []
    predicates = []
    for ttl in ttl_files:
        with open(ttl) as f:
            contents = f.readlines()
            for content in contents:
                content = content.strip()
                if content.startswith("@") and content not in prefixes:
                    prefixes.append(content)
                elif len(content) != 0:
                    predicate = content.split()[1]
                    if predicate not in predicates:
                        predicates.append(predicate)
    return prefixes, predicates


class QueryTranslation(dspy.Signature):
    original_query: str = dspy.InputField()
    rdf_prefixes: list[str] = dspy.InputField()
    triple_predicates: list[str] = dspy.InputField()
    translated_query: str = dspy.OutputField(
        desc="SPARQL query corresponding to user query for fetching requested data given RDF schema inferred from RDF prefixes and triple predicates"
    )


translate_query = dspy.Predict(QueryTranslation)


def fetch_data(query: str) -> Any:
    sparql = SPARQLWrapper("https://rdf.genenetwork.org/sparql/")
    sparql.setReturnFormat(JSON)
    prefixes, predicates = extract_schema(TTL_PATH)
    sparql_query = translate_query(
        original_query=query, rdf_prefixes=prefixes, triple_predicates=predicates
    ).get("translated_query")
    sparql.setQuery(sparql_query)
    return sparql.queryAndConvert()


fetch_data = dspy.Tool(
    name="fetch_data",
    desc="Fetch RDF data around GeneNetwork data through SPARQL",
    args={
        "query": {
            "type": "string",
            "desc": "SPARQL query to run to fetch relevant data",
        },
    },
    func=fetch_data,
)


class ReactSig(dspy.Signature):
    query: str = dspy.InputField()
    solution: ListInformation = dspy.OutputField(desc="The answer to the query")


class AISearch(dspy.Module):
    def __init__(self):
        super().__init__()
        self.tools = [fetch_data]

        self.react = dspy.ReAct(
            signature=ReactSig,
            tools=self.tools,
            max_iters=20,  # maximum number of steps for reasoning and tool calling
        )

    def forward(self, query: str):
        return self.react(query=query)
